----------------------------------------------------------------------
-- file:   message.sql
-- action: alter/create database package for WS CWS
-- usage:  connect as SYSTEM
--         run this script
--


set echo on

spool message.log

alter session set CURRENT_SCHEMA = CWS_DATA;

create or replace
package MESSAGE
is
  procedure ADD_MESSAGE
  ( MESSAGE_TO      in  varchar2
  , MESSAGE_FROM    in  varchar2
  , MESSAGE_SUBJECT in  varchar2
  , MESSAGE_CONTENT in  varchar2
  , ERROR_TEXT      out varchar2
  );

  procedure CLEAR_MESSAGES
  ( MESSAGE_TO      in  varchar2
  , ERROR_TEXT      out varchar2
  );

  procedure GET_MESSAGE
  ( MESSAGE_TO      in  varchar2
  , MESSAGE_FROM    out varchar2
  , MESSAGE_SUBJECT out varchar2
  , MESSAGE_CONTENT out varchar2
  , MESSAGE_NUMBER  out number
  , ENQUEUE_TIME    out date
  , ERROR_TEXT      out varchar2
  );
end MESSAGE;
/

show errors

create or replace
package body MESSAGE
is
  MESSAGE_NOT_FOUND  exception;

  pragma exception_init (MESSAGE_NOT_FOUND, -25228);

  procedure ADD_MESSAGE
  ( MESSAGE_TO      in  varchar2
  , MESSAGE_FROM    in  varchar2
  , MESSAGE_SUBJECT in  varchar2
  , MESSAGE_CONTENT in  varchar2
  , ERROR_TEXT      out varchar2
  )
  is
    ENQUEUE_OPTIONS      dbms_aq.enqueue_options_t;
    MESSAGE_ID           raw (16);
    MESSAGE_OBJECT       MESSAGE_TYPE;
    MESSAGE_PROPERTIES   dbms_aq.message_properties_t;
    NEXT_MESSAGE_NUMBER  number;
    TO_COUNT             number;
  begin
    select count(*)
     into  TO_COUNT
     from  APPLICATIONS
     where APPLICATION_ID = add_message.MESSAGE_TO;

    if TO_COUNT > 0 then
      select SEQ_MESSAGE_NUMBERS.nextval into NEXT_MESSAGE_NUMBER from dual;

      MESSAGE_OBJECT := MESSAGE_TYPE
      ( MESSAGE_NUMBER   => NEXT_MESSAGE_NUMBER
      , MESSAGE_TO       => add_message.MESSAGE_TO
      , MESSAGE_FROM     => add_message.MESSAGE_FROM
      , MESSAGE_SUBJECT  => add_message.MESSAGE_SUBJECT
      , MESSAGE_CONTENT  => add_message.MESSAGE_CONTENT
      );

      dbms_aq.ENQUEUE
      ( 'CWS_DATA.MESSAGE_QUEUE'
      , ENQUEUE_OPTIONS
      , MESSAGE_PROPERTIES
      , MESSAGE_OBJECT
      , MESSAGE_ID
      );

      ERROR_TEXT := null;
    else
      ERROR_TEXT := 'addressee not on record';
    end if;
  exception
    when others then
      ERROR_TEXT := SQLERRM;
  end ADD_MESSAGE;

  procedure CLEAR_MESSAGES
  ( MESSAGE_TO      in  varchar2
  , ERROR_TEXT      out varchar2
  )
  is
    ENQUEUE_TIME  date;
    MESSAGE_DATA  MESSAGE_TYPE := MESSAGE_TYPE (null,null,null,null,null);
  begin
    ERROR_TEXT := null;

    while ERROR_TEXT is null loop
      GET_MESSAGE
       ( MESSAGE_TO
       , MESSAGE_DATA.MESSAGE_FROM
       , MESSAGE_DATA.MESSAGE_SUBJECT
       , MESSAGE_DATA.MESSAGE_CONTENT
       , MESSAGE_DATA.MESSAGE_NUMBER
       , ENQUEUE_TIME
       , ERROR_TEXT
       );
    end loop;

    if ERROR_TEXT = 'no message found' then
      ERROR_TEXT := null;
    end if;
  end CLEAR_MESSAGES;

  procedure GET_MESSAGE
  ( MESSAGE_TO      in  varchar2
  , MESSAGE_FROM    out varchar2
  , MESSAGE_SUBJECT out varchar2
  , MESSAGE_CONTENT out varchar2
  , MESSAGE_NUMBER  out number
  , ENQUEUE_TIME    out date
  , ERROR_TEXT      out varchar2
  )
  is
    DEQUEUE_OPTIONS      dbms_aq.dequeue_options_t;
    MESSAGE_ID           raw (16);
    MESSAGE_OBJECT       MESSAGE_TYPE;
    MESSAGE_PROPERTIES   dbms_aq.message_properties_t;
  begin
    DEQUEUE_OPTIONS.NAVIGATION    := dbms_aq.FIRST_MESSAGE;
    DEQUEUE_OPTIONS.WAIT          := dbms_aq.NO_WAIT;
    DEQUEUE_OPTIONS.DEQ_CONDITION := 'tab.user_data.message_to = ''' || get_message.MESSAGE_TO
     || ''' and enq_time = (select min (enqueue_time) from messages where message_to = '''
     || get_message.MESSAGE_TO || ''' and dequeue_time is null)';

    dbms_aq.DEQUEUE
    ( 'CWS_DATA.MESSAGE_QUEUE'
    , DEQUEUE_OPTIONS
    , MESSAGE_PROPERTIES
    , MESSAGE_OBJECT
    , MESSAGE_ID
    );

    if MESSAGE_ID is not null then
      MESSAGE_FROM    := MESSAGE_OBJECT.MESSAGE_FROM;
      MESSAGE_SUBJECT := MESSAGE_OBJECT.MESSAGE_SUBJECT;
      MESSAGE_CONTENT := MESSAGE_OBJECT.MESSAGE_CONTENT;
      MESSAGE_NUMBER  := MESSAGE_OBJECT.MESSAGE_NUMBER;
      ENQUEUE_TIME    := MESSAGE_PROPERTIES.ENQUEUE_TIME;
    end if;

    ERROR_TEXT := null;
  exception
    when MESSAGE_NOT_FOUND then
      ERROR_TEXT := 'no message found';
    when others then
      ERROR_TEXT := SQLERRM;
  end GET_MESSAGE;
end MESSAGE;
/

show errors

spool off
